/*
 * Unit test of the shelllink functionality.
 *
 * Copyright 2003 Francois Gouget for CodeWeavers
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <assert.h>

#define COBJMACROS
#include <shlobj.h>
#include "wine/test.h"


inline static BOOL SHELL_OsIsUnicode(void)
{
    /* if high-bit of version is 0, the SHELL APIs are Unicode */
    return !(GetVersion() & 0x80000000);
}

HRESULT CreateLink(LPCWSTR lnkfile,
               LPCSTR path, LPCSTR relpath, LPCSTR args, LPCSTR workdir,
               LPCSTR description, LPCSTR iconpath, int iconindex)
{
    HRESULT hr;
    IShellLink* psl;
    IPersistFile* ppf;
    WCHAR widelink[MAX_PATH];
    
    MultiByteToWideChar(CP_ACP, 0, lnkfile, -1, widelink, MAX_PATH);
    hr=CoCreateInstance(&CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
                        &IID_IShellLinkA, (LPVOID*)&psl);
    assert(SUCCEEDED(hr));

    hr=IShellLinkA_QueryInterface(psl,&IID_IPersistFile,(LPVOID*)&ppf);
    if (path)
    {
	hr=IShellLinkA_SetPath(psl,path);
        ok(SUCCEEDED(hr),"SetPath failed: %lx\n",hr);
    }
    if (relpath)
    {
        hr=IShellLinkA_SetRelativePath(psl,relpath,0);
        ok(SUCCEEDED(hr),"SetRelativePath failed: %lx\n",hr);
    }
    if (args)
    {
        hr=IShellLinkA_SetArguments(psl,args);
        ok(SUCCEEDED(hr),"SetArguments failed: %lx\n",hr);
    }
    if (workdir)
    {
        hr=IShellLinkA_SetWorkingDirectory(psl,workdir);
        ok(SUCCEEDED(hr),"SetWorkingDirectory failed: %lx\n",hr);
    }
    if (description)
    {
        hr=IShellLinkA_SetDescription(psl,description);
        ok(SUCCEEDED(hr),"SetDescription failed: %lx\n",hr);
    }
    if (iconpath)
    {
        hr=IShellLinkA_SetIconLocation(psl,iconpath,iconindex);
        ok(SUCCEEDED(hr),"SetIconLocation failed: %lx\n",hr);
    }

    hr=IPersistFile_Save(ppf,widelink, TRUE);
    hr=IPersistFile_SaveCompleted(ppf, widelink);
    IPersistFile_Release(ppf);
    IShellLinkA_Release(psl);

    return hr;
}

HRESULT ReadLink(LPCWSTR linkfile,
                 LPSTR path, WIN32_FIND_DATA* find_data,
                 LPSTR args, LPSTR workdir,
                 LPSTR description, LPSTR iconpath, int* iconindex)
{
    HRESULT hr;
    IShellLink* psl;
    IPersistFile* ppf;
    WCHAR widelink[MAX_PATH];
    
    hr=CoCreateInstance(&CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
                        &IID_IShellLinkA, (LPVOID*)&psl);
    assert(SUCCEEDED(hr));

    hr=IShellLinkA_QueryInterface(psl,&IID_IPersistFile, (LPVOID*)&ppf);    
    if (SUCCEEDED(hr))
    {
	MultiByteToWideChar(CP_ACP, 0, (LPCSTR) linkfile, -1, widelink, MAX_PATH);	
        hr=IPersistFile_Load(ppf, widelink, STGM_READ);
	IShellLinkW_Resolve(psl, NULL, 0);
        if (SUCCEEDED(hr))
        {
      	    if (path)
            {
                strcpy(path,"xxx");
                hr=IShellLinkA_GetPath(psl, path, MAX_PATH, find_data, SLGP_RAWPATH);
                ok(SUCCEEDED(hr),"GetPath failed: %lx\n",hr);
            }
            if (args)
            {
                strcpy(args,"xxx");
                hr=IShellLinkA_GetArguments(psl, args, MAX_PATH);
                ok(SUCCEEDED(hr),"GetArguments failed: %lx\n",hr);
            }
            if (workdir)
            {
                strcpy(workdir,"xxx");
                hr=IShellLinkA_GetWorkingDirectory(psl, workdir, MAX_PATH);
                ok(SUCCEEDED(hr),"GetWorkingDirectory failed: %lx\n",hr);
            }
            if (description)
            {
                strcpy(description,"xxx");
                hr=IShellLinkA_GetDescription(psl, description, MAX_PATH);
                ok(SUCCEEDED(hr),"GetDescription failed: %lx\n",hr);
            }
            if (iconpath)
            {
                strcpy(iconpath,"xxx");
                *iconindex=0x12345678;
                hr=IShellLinkA_GetIconLocation(psl, iconpath, MAX_PATH,iconindex);
                ok(SUCCEEDED(hr),"GetIconLocation failed: %lx\n",hr);
            }
        }
        IPersistFile_Release(ppf);
    }
    IShellLinkA_Release(psl);

    return hr;
}

/* FIXME: This function does not yet test find_data and relpath */
void test_Links()
{
    HRESULT hr;
    char* link;
    WCHAR wlink[MAX_PATH];
    LPCOLESTR olink;
    char target[MAX_PATH];
    char path[MAX_PATH];
    char args[MAX_PATH];
    char workdir[MAX_PATH];
    char description[MAX_PATH];
    char iconpath[MAX_PATH];
    int iconindex;

    link="shortcut.lnk";
    if (SHELL_OsIsUnicode())
    {
        MultiByteToWideChar(CP_ACP,0,link,-1,wlink,sizeof(wlink)/sizeof(*wlink));
        olink=(LPCOLESTR)wlink;
    }
    else
    {
        olink=(LPCOLESTR)link;
    }

    /* Just create a basic link */
    GetModuleFileNameA(NULL,target,sizeof(target));
    strcat(target,".so");
    /* FIXME: For Winelib apps GetModuleFileNameA() points to an inexistant file */
    hr=CreateLink(olink,target,NULL,NULL,NULL,NULL,"",0);
    ok(SUCCEEDED(hr),"CreateLink failed hr=%lx\n",hr);
    hr=ReadLink(olink,path,NULL,args,workdir,description,iconpath,&iconindex);
    ok(SUCCEEDED(hr),"ReadLink failed hr=%lx\n",hr);
    if (SUCCEEDED(hr))
    {
        ok(lstrcmpiA(path,target)==0,"ReadLink got '%s' instead of '%s'\n",path,target);
        ok(*args=='\0',"ReadLink got args='%s'\n",args);
        ok(*workdir=='\0',"ReadLink got workdir='%s'\n",workdir);
        ok(*description=='\0',"ReadLink got description='%s'\n",description);
        ok(*iconpath=='\0',"ReadLink got iconpath='%s'\n",iconpath);
        ok(iconindex==0,"ReadLink got iconindex=%d\n",iconindex);
    }

    /* Make sure we can overwrite the previous link. Check the other fields */
    hr=CreateLink(olink,target,NULL,"shelllink","c:\\Temp","My Description",target,2);
    ok(SUCCEEDED(hr),"CreateLink failed hr=%lx\n",hr);
    hr=ReadLink(olink,path,NULL,args,workdir,description,iconpath,&iconindex);
    ok(SUCCEEDED(hr),"ReadLink failed hr=%lx\n",hr);
    if (SUCCEEDED(hr))
    {
        ok(lstrcmpiA(path,target)==0,"ReadLink got '%s' instead of '%s'\n",path,target);
        ok(strcmp(args,"shelllink")==0,"ReadLink got args='%s'\n",args);
        ok(lstrcmpiA(workdir,"c:\\Temp")==0,"ReadLink got workdir='%s'\n",workdir);
        ok(lstrcmpiA(description,"My Description")==0,"ReadLink got description='%s'\n",description);
        ok(lstrcmpiA(iconpath,target)==0,"ReadLink got iconpath='%s'\n",iconpath);
        ok(iconindex==2,"ReadLink got iconindex=%d\n",iconindex);
    }
    DeleteFile(link);

    /* See if we can link to a non-existent file */
    hr=CreateLink(olink,"C:\\non-existent.file",NULL,"shelllink","c:\\Temp","My Description","non-existent.file",2);
    ok(SUCCEEDED(hr),"CreateLink failed hr=%lx\n",hr);
    hr=ReadLink(olink,path,NULL,args,workdir,description,iconpath,&iconindex);
    ok(SUCCEEDED(hr),"ReadLink failed hr=%lx\n",hr);
    if (SUCCEEDED(hr))
    {
        ok(lstrcmpiA(path,"C:\\non-existent.file")==0,"ReadLink got '%s' instead of '%s'\n",path,"C:\\non-existent.file");
        ok(strcmp(args,"shelllink")==0,"ReadLink got args='%s'\n",args);
        ok(lstrcmpiA(workdir,"c:\\Temp")==0,"ReadLink got workdir='%s'\n",workdir);
        ok(lstrcmpiA(description,"My Description")==0,"ReadLink got description='%s'\n",description);
        ok(lstrcmpiA(iconpath,"non-existent.file")==0,"ReadLink got iconpath='%s'\n",iconpath);
        ok(iconindex==2,"ReadLink got iconindex=%d\n",iconindex);
    }

    DeleteFile(link);
}

START_TEST(shelllink)
{
    CoInitialize(NULL);

    test_Links();

    CoUninitialize();
}
